<!DOCTYPE html>
<html>
{% include 'head.html' %}
<body>
    {% include 'header.html' %}
    {% include 'sign-out.html' %}
    <div id="container">
        <div id="about">
            <div id="start-text">
            Something went wrong while trying to finalize your subscription.
            Please try again. If the issue persists, please contact us through
            <a href="mailto:professional@cogsci.nl">sales@sigmundai.eu</a>.
            </div>
        </div>
    {% include 'footer.html' %}
    </div>
</body>
</html>
